@@ -3,6 +3,7 @@ from __future__ import annotations
 import abc
 import enum
 import json
+import logging
 import uuid
 from dataclasses import dataclass, field
 from typing import Iterable, Optional, Type, TypeVar
@@ -10,6 +11,7 @@ from typing import Iterable, Optional, Type, TypeVar
 from google.protobuf.duration_pb2 import Duration
 from google.protobuf.empty_pb2 import Empty as GEmpty
 
+from esdb.client.exceptions import ClientException, StreamNotFound, WrongExpectedVersion
 from esdb.generated.shared_pb2 import UUID, Empty, StreamIdentifier
 from esdb.generated.streams_pb2 import (
     AppendReq,
@@ -25,6 +27,8 @@ from esdb.generated.streams_pb2 import (
 )
 from esdb.generated.streams_pb2_grpc import StreamsStub
 
+logger = logging.getLogger(__name__)
+
 
 @enum.unique
 class StreamState(enum.Enum):
@@ -81,7 +85,7 @@ class ContentType(enum.Enum):
 
 
 @dataclass
-class ReadResult:
+class ReadEvent:
     id: str
     stream_name: str
     prepare_position: int
@@ -92,8 +96,8 @@ class ReadResult:
     data: dict | bytes
 
     @staticmethod
-    def from_response(response: ReadResp) -> "ReadResult":
-        return ReadResult(
+    def from_response(response: ReadResp) -> ReadEvent:
+        return ReadEvent(
             id=response.event.event.id.string,
             stream_name=response.event.event.stream_identifier.stream_name.decode(),
             metadata=response.event.event.metadata,
@@ -109,6 +113,17 @@ class ReadResult:
         )
 
 
+@dataclass
+class SubscriptionConfirmed:
+    subscription_id: str
+
+
+@dataclass
+class Checkpoint:
+    commit_position: int
+    prepare_position: int
+
+
 ProposedMessageType = TypeVar("ProposedMessageType", BatchAppendReq.ProposedMessage, AppendReq.ProposedMessage)
 
 
@@ -214,10 +229,8 @@ class StreamsBase(abc.ABC):
 
     @staticmethod
     def _process_append_response(response: AppendResp) -> AppendResult:
-        if response.HasField("wrong_expected_version"):
-            raise Exception(f"TODO: wrong expected version: {response.wrong_expected_version}")
-        if not response.HasField("success"):
-            raise Exception(f"TODO: {response}")
+        if response.WhichOneof("result") == "wrong_expected_version":
+            raise WrongExpectedVersion(response.wrong_expected_version)
         return AppendResult.from_response(response)
 
     @abc.abstractmethod
@@ -289,10 +302,20 @@ class StreamsBase(abc.ABC):
         )
 
     @staticmethod
-    def _process_read_response(response: ReadResp) -> ReadResult:
-        if response.HasField("stream_not_found"):
-            raise Exception("TODO: Stream not found")
-        return ReadResult.from_response(response)
+    def _process_read_response(response: ReadResp) -> ReadEvent | SubscriptionConfirmed | Checkpoint:
+        content = response.WhichOneof("content")
+        if content == "event":
+            return ReadEvent.from_response(response)
+        if content == "confirmation":
+            return SubscriptionConfirmed(response.confirmation.subscription_id)
+        if content == "checkpoint":
+            return Checkpoint(
+                commit_position=response.checkpoint.commit_position,
+                prepare_position=response.checkpoint.prepare_position,
+            )
+        if content == "stream_not_found":
+            raise StreamNotFound(response.stream_not_found)
+        raise ClientException(f"Got unexpected response {content}: {getattr(response, content)}")
 
     @abc.abstractmethod
     def read(
@@ -301,7 +324,7 @@ class StreamsBase(abc.ABC):
         count: int,
         backwards: bool = False,
         revision: int | None = None,
-    ) -> ReadResult:
+    ) -> ReadEvent:
         ...
 
     @staticmethod
@@ -322,8 +345,9 @@ class StreamsBase(abc.ABC):
         )
 
     @staticmethod
-    def _process_delete_response(response: DeleteResp) -> DeleteResult:
-        return DeleteResult.from_response(response)
+    def _process_delete_response(response: DeleteResp) -> Optional[DeleteResult]:
+        has_position = response.WhichOneof("position_option") == "position"
+        return DeleteResult.from_response(response) if has_position else None
 
     @abc.abstractmethod
     def delete(
@@ -349,8 +373,9 @@ class StreamsBase(abc.ABC):
         )
 
     @staticmethod
-    def _process_tombstone_response(response: TombstoneResp) -> TombstoneResult:
-        return TombstoneResult.from_response(response)
+    def _process_tombstone_response(response: TombstoneResp) -> Optional[TombstoneResult]:
+        has_position = response.WhichOneof("position_option") == "position"
+        return TombstoneResult.from_response(response) if has_position else None
 
     @abc.abstractmethod
     def tombstone(
@@ -363,30 +388,24 @@ class StreamsBase(abc.ABC):
         stream: str,
         messages: Iterable[Message],
         stream_state: StreamState = StreamState.ANY,
-        revision: int | None = None,
         correlation_id: None | uuid.UUID = None,
         deadline_ms: None | int = None,
+        stream_position: Optional[int] = None,
     ) -> Iterable[BatchAppendReq]:
         correlation_id_ = UUID(string=str(correlation_id or uuid.uuid4()))
-        options: dict[str, int | GEmpty | None]
-        if revision is not None:
-            # Append at specified revision
-            options = {"revision": revision}
-        else:
-            options = {v.value: None for v in StreamState}
-            options[stream_state.value] = GEmpty()
-
+        options = {v.value: None for v in StreamState}
+        options[stream_state.value] = GEmpty()
         if deadline_ms is not None:
             options["deadline"] = Duration(nanos=deadline_ms * 1000000)
 
         yield BatchAppendReq(
             correlation_id=correlation_id_,
             options=BatchAppendReq.Options(
-                # stream_position=0,  # TODO: wtf is this? probably expected position
+                stream_position=stream_position,  # this doesn't seem to do anything
                 stream_identifier=StreamIdentifier(stream_name=stream.encode()),
                 **options,
             ),
-            is_final=False,  # This needs to be off for options
+            is_final=False,
         )
         yield BatchAppendReq(
             correlation_id=correlation_id_,
@@ -396,10 +415,10 @@ class StreamsBase(abc.ABC):
 
     @staticmethod
     def _process_batch_append_response(response: BatchAppendResp) -> BatchAppendResult:
-        if response.HasField("error"):
-            raise Exception(f"TODO: got error {response.error}")
-        if not response.HasField("success"):
-            raise Exception(f"TODO: {response}")
+        result = response.WhichOneof("result")
+        if result == "error":
+            # For some reason ES uses google.rpc.Status here instead of more meaningful error.
+            raise ClientException(f"Append failed with {response.error.message} and code {response.error.code}")
         return BatchAppendResult.from_response(response)
 
     @abc.abstractmethod
